/**
 * class for parsing a URL (and its query string)
 * @class
 */
QueryString = function(queryString)
{
   this.parameterMap = new Object();
   
   if (queryString)
   {
      this.parse(queryString);
   }
};

QueryString.create = function(queryString)
{
   if (!queryString)
   {
      return new QueryString();
   }
   
   if (queryString.constructor == QueryString)
   {
      return queryString;
   }
   
   return new QueryString(queryString.toString());
};

QueryString.prototype.parse = function(queryString)
{
   var parameters = queryString.split("&");
      
   for (i=0; i<parameters.length; i++)
   {
      var param = parameters[i];
      pos = param.indexOf("=");
      
      var name, value;
      
      if (pos == -1)
      {
         name = param;
         value = null;
      }
      else
      {
         name = param.substring(0, pos);
         value = param.substring(pos+1);
      }
      
      if (name == "")
      {
         continue;
      }
      
      this.addParameter(name, value);
   }
};

/**
 * Returns the parameter map representing this 
 */
QueryString.prototype.getParameters = function()
{
   return this.parameterMap;
};

/**
 * removes a parameter from the query string
 * @param {String} name parameter name
 */
QueryString.prototype.removeParameter = function(name)
{
   delete this.parameterMap[name];
};

/**
 * sets the value of a query string parameter
 * @param {String} name parameter name
 * @param {Strign} value parameter value
 */
QueryString.prototype.setParameter = function(name, value)
{
   if (value === null)
   {
      this.removeParameter(name);
   }
   else
   {
      this.parameterMap[name] = value;
   }
};

/**
 * sets the value of a query string parameter
 * @param {String} name parameter name
 * @param {Strign} value parameter value
 */
QueryString.prototype.addParameter = function(name, value)
{
   var existingValue = this.parameterMap[name];
   
   if (existingValue !== undefined)
   {
      if (existingValue.constructor === Array)
      {
         existingValue.push(value);
         value = existingValue;
      }
      else
      {
         value = [existingValue, value];
      }
   }
   
   this.parameterMap[name] = value;
};

/**
 * retrieves a value of parameter from the query string
 * @param {String} name parameter name
 */
QueryString.prototype.getParameter = function(name)
{
   var value = this.parameterMap[name];
   if (value === undefined)
   {
      return null;
   }
   
   if (value.constructor === Array)
   {
      return value[0];
   }
   else
   {
      return value;
   }
};

/**
 * Converts the QueryString to its string representation without encoding parameter values. Call this if you know the value are
 * already encoded.
 * @return {String} string representation of QueryString
 */
QueryString.prototype.toUnencodedString = function()
{
   return getString(this.parameterMap, false);
};

/**
 * Converts the QueryString to its string representation
 * 
 * @return {String} string representation of QueryString
 */
QueryString.prototype.toString = function()
{
   return getString(this.parameterMap, true);
};

getString = function(parameterMap, encodeValues)
{
   var qs = "";
   
   for (var name in parameterMap)
   {
      var value = parameterMap[name];
      
      if (value == null)
      {
         qs += name + "&";
      }
      else if (value.constructor === Array)
      {
         for (var i = 0; i < value.length; i++)
         {
            qs += name + "=" + (encodeValues ? encodeURIComponent(value[i]) : value[i]) + "&";
         }
      }
      else
      {
         qs += name + "=" + (encodeValues ? encodeURIComponent(value) : value) + "&";
      }
   }
   
   return qs;
};

QueryString.prototype.toDecodedString = function()
{
	var qs="";
	for (var name in this.parameterMap)
	{
		var value = this.parameterMap[name];
		
		if (value == null)
		{
			qs += name + "&";
		}
	    else if (value.constructor === Array)
	    {
	       for (var i = 0; i < value.length; i++)
	       {
	          qs += name + "=" + decodeURIComponent(value[i]) + "&";
	       }
	    }
	    else
	    {
	       qs += name + "=" + decodeURIComponent(value) + "&";
	    }
	}
	
	return qs;
};
