
/***********************************************************************************/
/* Copyright (c) 2012, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_newctr_detail                                 */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       Transform the RR data for e-filing (detail record level)         */
/* PARAMETERS:                                                                     */
/*                rr_subset_dsn: name of the table containing to be processed      */
/*                               rr_rk                                             */
/*                form_config_rk: form config key                                  */
/*                                                                                 */
/* NOTE:          Input tables:                                                    */
/*                  &rr_subset_dsn                                                 */
/*                Output tables:                                                   */
/*                  x_transmitter             (1A)                                 */  
/*                  t_zx_institution          (2B)                                 */  
/*                  t_zx_rr                   (3A 2A 9A)                           */ 
/*                  r_x_foreign_cash_in       (3B)                                 */
/*                  r_x_foreign_cash_out      (3C)                                 */
/*                  t_zx_subject              (4A)                                 */
/*                  r_x_account_cash_in       (4B)                                 */
/*                  r_x_account_cash_out      (4C)                                 */
/*                  These output tables will be transformed to submission ready    */
/*                  data by ecmrr_efile_fincen_newctr_sum                          */
/***********************************************************************************/

%macro ecmrr_efile_fincen_newctr_detail(rr_subset_dsn=rr_subset,form_config_rk=202);
                                          
%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

%let sucessYN=Y;

/***********************************************/
/* Get RR records                              */
/* and transpose them to rectangular structure */
/***********************************************/

   %ecm_create_label_fmt; 

    %if &successYN=N %then %goto mEXIT;

%let table_wh="X_FOREIGN_CASH_IN" "X_FOREIGN_CASH_OUT" "X_INSTITUTION" 
 "RR" "X_ACCOUNT_CASH_IN" "X_ACCOUNT_CASH_OUT" "X_PERSON_INVOLVED" "X_TRAN_METHOD"
;

    /* To improve performance, subset the data first */

    %macro ecm_extract_udf_value(udf_table_nm=);    
    proc sql;
    create table &udf_table_nm as
    select a.*
    from   ECM_DB.&udf_table_nm as a  
         , ECM_DB.RR_live as b
    where a.RR_rk=b.RR_rk 
          and a.valid_from_dttm=b.valid_from_dttm 
          and a.RR_rk in (select distinct RR_rk
           from rr_subset) ;
    quit;
    %mend;

    proc sql;
        create table rr_live as
        select * from ecm_db.rr_live
        where rr_rk in (select distinct RR_rk
           from rr_subset) ;
    quit;

    %ecm_extract_udf_value(udf_table_nm=RR_UDF_CHAR_VALUE);
    %ecm_extract_udf_value(udf_table_nm=RR_UDF_DATE_VALUE);
    %ecm_extract_udf_value(udf_table_nm=RR_UDF_NUM_VALUE);
    %ecm_extract_udf_value(udf_table_nm=RR_UDF_LGCHR_VIEW);

   /* Generate a list of report keys for future filtering */
    proc sql noprint;
         select distinct strip(put(rr_rk,best.))
                         into : rr_rk_list separated by  ' '
         from rr_live
         ;
     
    quit;
 
  
    /*Comment out the old way */

   %*ecm_pivot_datatype(datatype=rr,in_lib=ECM_DB,out_lib=WORK,subset_dsn=&rr_subset_dsn,table_wh=&table_wh);

    /* Transform the data into report ready format using the extracted data in the WORK library. */
   %ecm_pivot_datatype(datatype=rr,in_lib=WORK,out_lib=WORK,datatype_rk=&rr_rk_list,table_wh=&table_wh);
       
   %if &successYN=N %then %goto mEXIT;

/***********************************************/
/* Prepare Transmitter data   (1A)             */
/***********************************************/
    
%ecmrr_efile_fincen_trnsmttr_rec(outdsn=x_transmitter,efile_rk=&efile_rk,form_type_cd=newctr);
        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
   


/*********************************************************************************************/
/* Concatenate the following codes into a string for efiling                                 */
/* X_TRAN_METHOD_TYPE_CD (5 max) - transacton type code (item 24) 3A POS 31                  */
/*********************************************************************************************/

%ecmrr_fincen_concat_cd(indsn=r_x_tran_method
                            ,outdsn=t_zx_tran_method
                            ,byvar=rr_rk
                            ,invar=x_tran_method_type_cd
                            ,outvar=zx_tran_method_type
                            ,maxsize=5);  
 

/********************************************/
/* No special processing for 3B-3C          */
/* The detail tables will be use are        */
/* 3B: R_X_FOREIGN_CASH_IN_                 */
/* 3C: R_X_FOREIGN_CASH_OUT                 */
/********************************************/

 


/********************************************/
/* Prepare t_zx_person_involved for 4A      */
/********************************************/

 
data t_zx_person_involved;
     set r_x_person_involved;
     rename row_no=z_subject_seq_no
     ;
run;
 


%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;



/********************************************************/
/* Assign subject seq number to subject related tables  */
/********************************************************/



proc sql noprint; 
     create table t_zx_account_cash_in as
     select a.*
            ,b.z_subject_seq_no
     from r_x_account_cash_in as a,
          t_zx_person_involved as b
     where a.x_person_involved_rk=b.x_person_involved_rk
     ;
     create table t_zx_account_cash_out as
     select a.*, b.z_subject_seq_no
     from r_x_account_cash_out as a,
          t_zx_person_involved as b
     where a.x_person_involved_rk=b.x_person_involved_rk
     ;


%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end; 
 
 
/*************************************************/
/* Count number of subjects for each report (9A) */
/*************************************************/

 
proc freq data=t_zx_person_involved ;
    table rr_rk /out=t_z_subject_cnt (drop=percent rename=(count=z_subject_cnt)) noprint;
run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
 


/*********************************************************/
/* Count number of cash in accounts for each report (9A) */
/*********************************************************/

 
proc freq data=t_zx_account_cash_in;
    table rr_rk /out=t_z_cash_in_acct_cnt (drop=percent rename=(count=z_cash_in_acct_cnt)) noprint;
run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;




/*********************************************************/
/* Count number of cash out accounts for each report (9A) */
/*********************************************************/

 
proc freq data=t_zx_account_cash_out;
    table rr_rk /out=t_z_cash_out_acct_cnt (drop=percent rename=(count=z_cash_out_acct_cnt)) noprint;
run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/********************************************/
/* Prepare T_ZX_RR (3A)                     */
/********************************************/


data t_zx_rr;
    merge rr_pivot (in=inrr)
          t_z_subject_cnt (in=in_subject_cnt)
          t_z_cash_in_acct_cnt (in=in_cash_in_acct_cnt)
          t_z_cash_out_acct_cnt (in=in_cash_out_acct_cnt )
          t_zx_tran_method (drop=row_no)
          
    ;
    by rr_rk;
    if inrr;
    if not in_subject_cnt then z_subject_cnt=0;
    if not in_cash_in_acct_cnt then z_cash_in_acct_cnt=0;
    if not in_cash_out_acct_cnt then z_cash_out_acct_cnt=0;
    x_filed_dt=today();
    
run;




%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;



%mExit:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;

%mend;
 
