
/***********************************************************************************/
/* Copyright (c) 2014, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_sar_detail                                    */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       Transform the RR data for e-filing (detail record level)         */
/* PARAMETERS:                                                                     */
/*                rr_subset_dsn: name of the table containing to be processed      */
/*                               rr_rk                                             */
/*                form_config_rk: form config key                                  */
/*                                                                                 */
/* NOTE:          Input tables:                                                    */
/*                  &rr_subset_dsn                                                 */
/*                Output tables:                                                   */
/*                  x_transmitter             (1A)                                 */  
/*                  t_zx_institution          (2B)                                 */ 
/*                  r_x_branch                (2C)                                 */ 
/*                  t_zx_rr                   (3A, 2A, 9B, 9Z)                     */ 
/*                  r_x_commodity             (3B)                                 */
/*                  r_x_product               (3C)                                 */
/*                  r_x_market                (3D)                                 */
/*                  r_x_ip_address            (3E)                                 */
/*                  r_x_cusip                 (3F)                                 */
/*                  t_zx_subject              (4A)                                 */
/*                  r_x_subject_address       (4B)                                 */
/*                  r_x_subject_id            (4C)                                 */
/*                  r_x_subject_alternate_nm  (4D)                                 */
/*                  r_x_subject_telephone_no  (4E)                                 */
/*                  r_x_subject_email_address (4F)                                 */
/*                  r_x_subject_url_address   (4G)                                 */
/*                  t_zx_subject_relation_inst(4H)                                 */
/*                  t_zx_subject_inst_acct    (4I)                                 */
/*                  t_z_sar_narrative_desc    (5A)                                 */
/*                  t_z_sum_rr                (9A)                                 */
/*                  These output tables will be transformed to submission ready    */
/*                  data by ecmrr_efile_fincen_sar_sum                             */
/***********************************************************************************/

%macro ecmrr_efile_fincen_sar_detail(rr_subset_dsn=rr_subset,form_config_rk=201);
                                          
%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

%let sucessYN=Y;

/***********************************************/
/* Get RR records                              */
/* and transpose them to rectangular structure */
/***********************************************/

   %ecm_create_label_fmt; 

    %if &successYN=N %then %goto mEXIT;

%let table_wh="X_BRANCH" "X_COMMODITY" "X_CUSIP"  "X_FILING_INST_SF_TYPE" "X_INSTITUTION" "X_INST_SF_TYPE"
"X_IP_ADDRESS" "X_MARKET" "X_PRODUCT" "X_SAR_CASINO_TYPE" "X_SAR_FRAUD_TYPE" "X_SAR_ID_TYPE" "X_SAR_INSTRUMENT_TYPE"
"X_SAR_INSURANCE_TYPE" "X_SAR_ML_TYPE" "X_SAR_MORTGAGE_TYPE" "X_SAR_OTHER_TYPE" "X_SAR_PROD_TYPE" "X_SAR_SF_TYPE"
"X_SAR_STRUCT_TYPE" "X_SAR_TERRORIST_TYPE" "X_SAR_TYPE" "X_SUBJECT" "X_SUBJECT_ACCT_INST" "X_SUBJECT_ADDRESS"
"X_SUBJECT_ALTERNATE_NM" "X_SUBJECT_EMAIL_ADDRESS" "X_SUBJECT_ID" "X_SUBJECT_INST_ACCT" "X_SUBJECT_RELATION"
"X_SUBJECT_RELATION_INST" "X_SUBJECT_TELEPHONE_NO" "X_SUBJECT_URL_ADDRESS" "RR"
;

    /* To improve performance, subset the data first */

    %macro ecm_extract_udf_value(udf_table_nm=);    
    proc sql;
    create table &udf_table_nm as
    select a.*
    from   ECM_DB.&udf_table_nm as a  
         , ECM_DB.RR_live as b
    where a.RR_rk=b.RR_rk 
          and a.valid_from_dttm=b.valid_from_dttm 
          and a.RR_rk in (select distinct RR_rk
           from rr_subset) ;
    quit;
    %mend;

    proc sql;
        create table rr_live as
        select * from ecm_db.rr_live
        where rr_rk in (select distinct RR_rk
           from rr_subset) ;
    quit;

    %ecm_extract_udf_value(udf_table_nm=RR_UDF_CHAR_VALUE);
    %ecm_extract_udf_value(udf_table_nm=RR_UDF_DATE_VALUE);
    %ecm_extract_udf_value(udf_table_nm=RR_UDF_NUM_VALUE);
    %ecm_extract_udf_value(udf_table_nm=RR_UDF_LGCHR_VIEW);

   /* Generate a list of report keys for future filtering */
    proc sql noprint;
         select distinct strip(put(rr_rk,best.))
                         into : rr_rk_list separated by  ' '
         from rr_live
         ;
     
    quit;
 
  
    /*Comment out the old way */

   %*ecm_pivot_datatype(datatype=rr,in_lib=ECM_DB,out_lib=WORK,subset_dsn=&rr_subset_dsn,table_wh=&table_wh);

    /* Transform the data into report ready format using the extracted data in the WORK library. */
   %ecm_pivot_datatype(datatype=rr,in_lib=WORK,out_lib=WORK,datatype_rk=&rr_rk_list,table_wh=&table_wh);

     
    
   %if &successYN=N %then %goto mEXIT;


/***********************************************/
/* Prepare Transmitter data   (1A)             */
/***********************************************/
    
%ecmrr_efile_fincen_trnsmttr_rec(outdsn=x_transmitter,efile_rk=&efile_rk,form_type_cd=SAR);
        
%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
    
 
  
/*********************************************************************************************/
/* Concatenate the following codes into a string for efiling                                 */
/* X_FILING_INST_SF_TYPE_CD (13 max) - filing institution SF types (item 83)   2A POS 232    */
/* X_INST_SF_TYPE_CD (10 max) - Institution SF types (item 50)   2B POS 120                  */
/*********************************************************************************************/

%ecmrr_fincen_concat_cd(indsn=r_x_filing_inst_sf_type
                            ,outdsn=t_zx_filing_inst_sf_type
                            ,byvar=rr_rk
                            ,invar=x_filing_inst_sf_type_cd
                            ,outvar=zx_filing_inst_sf_type
                            ,maxsize=13);

%if &successYN=N %then %goto mExit;

%ecmrr_fincen_concat_cd(indsn=r_x_inst_sf_type
                            ,outdsn=t_zx_inst_sf_type
                            ,byvar=rr_rk x_inst_rk
                            ,invar=x_inst_sf_type_cd
                            ,outvar=zx_inst_sf_type
                            ,inothervar=x_inst_sf_type_oth_txt
                            ,otherVal=Z
                            ,outothervar=zx_inst_sf_type_oth_txt
                            ,maxsize=10
                            ,sortedYN=N
                            ,sortby=rr_rk x_inst_rk
                            );

%if &successYN=N %then %goto mExit;


%ecmrr_fincen_concat_sar_type_cd;




/*************************************************************/
/* Transpose subject relations into one                      */
/* concatenated field for each subject and institution       */
/* up to 12 subject/institution relation can be reported (4H)*/
/*************************************************************/

%ecmrr_fincen_concat_cd(indsn=r_x_subject_relation
                            ,outdsn=t_zx_subject_relation
                            ,byvar=rr_rk x_subject_relation_inst_rk
                            ,invar=x_subject_relation_cd
                            ,outvar=zx_subject_relation
                            ,inothervar=x_subject_relation_oth_txt
                            ,otherVal=Z
                            ,outothervar=zx_subject_relation_oth_txt
                            ,maxsize=12
                            ,sortedYN=N
                            ,sortby=rr_rk x_subject_relation_inst_rk);
                          
 
%if &successYN=N %then %goto mExit;
 

/********************************************/
/* Break up long narrative column into      */
/* $4000 strings                            */
/* up to 10 rows are reported  (5A)         */
/********************************************/                           
%ecmrr_fincen_narrative(indsn=rr_pivot
                            ,outdsn=t_z_sar_narrative_desc
                            ,idvar=rr_rk 
                            ,invar=x_sar_narrative_desc
                            ,outvar=z_sar_narrative_desc
                            ,outseqvar=ROW_NO 
                            ,outsize=850
                            ,outseqmax=20
                            );

%if &successYN=N %then %goto mExit;


 
/********************************************/
/* Prepare T_ZX_INSTITUION for 2B           */
/********************************************/
proc sql;
    create table t_zx_institution as
    select distinct a.*, b.zx_inst_sf_type, b.zx_inst_sf_type_oth_txt
    from  r_x_institution (rename=(row_no=z_institution_seq_no)) as a
    left join   t_zx_inst_sf_type as b
    on a.rr_rk=b.rr_rk and a.x_inst_rk=b.x_inst_rk
     ;
run;


/********************************************/
/* Prepare T_ZX_RR (3A, 2A, 9B, 9Z)         */
/********************************************/
 

data t_zx_rr;
    merge rr_pivot (in=inrr)
          r_x_sar_type        
          t_zx_filing_inst_sf_type 
    ;
    by rr_rk;
    if inrr;
    x_filed_dt=today();
         
run;




%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;


/********************************************/
/* No special processing for 3B-3F          */
/* The detail tables will be use are        */
/* 3B: R_X_COMMODITY                        */
/* 3C: R_X_PRODUCT                          */
/* 3D: R_X_MARKET                           */
/* 3E: R_X_IP_ADDRESS                       */
/* 3F: R_X_CUSIP                            */
/********************************************/
 


/********************************************/
/* Prepare t_zx_subject for 4A                */
/********************************************/
 
data t_zx_subject;
     set r_x_subject;
     rename row_no=z_subject_seq_no
     ;
run;
 


%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;


/********************************************/
/* No special processing for 4B-4G          */
/* The detail tables will be use are        */
/* 4B: R_X_SUBJECT_ADDRRESS                 */
/* 4C: R_X_SUBJECT_ID                       */
/* 4D: R_X_SUBJECT_ALTERNATE_NM             */
/* 4E: R_X_SUBJECT_TELEPHONE_NO             */
/* 4F: R_X_SUBJECT_EMAIL_ADDRESS            */
/* 4G: R_X_SUBJECT_URL_ADDRESS              */
/********************************************/


/*********************************************/
/* Prepare T_ZX_SUBJECT_RELATION_INST for 4H */
/*********************************************/
 
proc sql noprint;
create table t_zx_subject_relation_inst as
select distinct a.*, b.zx_subject_relation, b.zx_subject_relation_oth_txt
from 
(select c.*, d.x_inst_tin_no  
 from   r_x_subject_relation_inst as c
 left join   r_x_institution as d
 on c.rr_rk=d.rr_rk and c.x_inst_rk=d.x_inst_rk 
) as a
left join t_zx_subject_relation as b 
on a.rr_rk=b.rr_rk and a.x_subject_relation_inst_rk=b.x_subject_relation_inst_rk
;



%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;



/*********************************************/
/* Prepare T_ZX_SUBJECT_INST_ACCT for 4I     */
/*********************************************/

proc sql noprint;
create table t_zx_subject_inst_acct as
select a.*, b.x_acct_id, b.x_acct_closed_flg
from r_x_subject_acct_inst as a
left join     r_x_subject_inst_acct as b 
on a.rr_rk=b.rr_rk and a.x_subject_rk=b.x_subject_rk and a.x_acct_inst_rk=b.x_acct_inst_rk
;



%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;


/********************************************/
/* Prepare T_Z_SUM_RR for 9A                */
/********************************************/

/*****************************************************************************/
/* Count number of institutions where activity occured for each report (9A)  */
/*****************************************************************************/
 
proc freq data=r_x_institution ;
    table rr_rk /out=t_z_institution_cnt (drop=percent rename=(count=z_institution_cnt)) noprint;
run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;



/***************************************************************************/
/* Count number of branches where activity occurred for each report  (9A)  */
/***************************************************************************/
 
proc freq data=r_x_branch ;
    table rr_rk /out=t_z_branch_cnt (drop=percent rename=(count=z_branch_cnt)) noprint;
run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;
        
 
/*************************************************/
/* Count number of subjects for each report (9A) */
/*************************************************/

 
proc freq data=t_zx_subject ;
    table rr_rk /out=t_z_subject_cnt (drop=percent rename=(count=z_subject_cnt)) noprint;
run;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

/**********************************************/
/* Count number of explanation lines (9A)     */
/**********************************************/

   proc sql;
        create table t_z_sar_narrative_cnt as
        select distinct rr_rk, count(*) as z_sar_narrative_cnt
        from t_z_sar_narrative_desc
        group by rr_rk
        order by rr_rk;
    quit;

%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;


data t_z_sum_rr;
    merge rr_pivot (in=inrr keep=rr_rk x_sar_attachment_nm) 
          t_z_subject_cnt (in=in_subject_cnt)
          t_z_institution_cnt (in=in_institution_cnt)
          t_z_branch_cnt (in=in_branch_cnt)
          t_z_sar_narrative_cnt (in=in_sar_narrative_cnt)
    ;
    by rr_rk;
    if inrr;
    if not in_subject_cnt then z_subject_cnt=0;
    if not in_institution_cnt then z_institution_cnt=0;
    if not in_branch_cnt then z_branch_cnt=0;
    if not in_sar_narrative_cnt then z_sar_narrative_cnt=0;
    
    if x_sar_attachment_nm ne '' then z_attachment_cnt=1; 
    else z_attachment_cnt=0;
    drop x_sar_attachment_nm;
    
run;




%if &syserr > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;



%mExit:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;

%mend;
 
